"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var stringEmpty = "", toString = Object.prototype.toString, core_hasOwn = Object.prototype.hasOwnProperty, slice = Array.prototype.slice;
function testObject(obj) {
    if (obj.constructor &&
        !core_hasOwn.call(obj, "constructor") &&
        !core_hasOwn.call(obj.constructor.prototype, "isPrototypeOf")) {
        return false;
    }
}
var Lib = /** @class */ (function () {
    function Lib() {
    }
    Lib.noop = function () { };
    Lib.hasOwnProp = function (obj, prop) {
        return core_hasOwn.call(obj, prop);
    };
    Lib.trace = function (e) {
        console.error && console.error(e.stack || e.message || e + '');
    };
    Lib.isType = function (typename, value) {
        //typename:String, Array, Boolean, Object, RegExp, Date, Function,Number //兼容
        //typename:Null, Undefined,Arguments    //IE不兼容
        return toString.apply(value) === '[object ' + typename + ']';
    };
    Lib.toStr = function (p) {
        return Lib.isNull(p) ? '' : p.toString();
    };
    Lib.isUndefined = function (obj) {
        ///<summary>是否定义</summary>
        return (typeof (obj) === "undefined" || obj === undefined);
    };
    Lib.isNull = function (obj) {
        ///<summary>是否Null</summary>
        return (obj === null || Lib.isUndefined(obj));
    };
    Lib.isBoolean = function (obj) {
        return Lib.isType("Boolean", obj);
    };
    Lib.isNullEmpty = function (s) {
        return (Lib.isNull(s) || s === stringEmpty);
    };
    Lib.isFunction = function (fun) {
        return Lib.isType("Function", fun);
    };
    Lib.isNumeric = function (n) {
        //return cmpx.isType("Number", n) && !isNaN(n) && isFinite(n);;
        return !isNaN(parseFloat(n)) && isFinite(n);
    };
    Lib.isString = function (obj) {
        return Lib.isType("String", obj);
    };
    Lib.isObject = function (obj) {
        return obj && Lib.isType("Object", obj)
            && !Lib.isElement(obj) && !Lib.isWindow(obj); //IE8以下isElement, isWindow认为Object
    };
    Lib.isPlainObject = function (obj) {
        if (!Lib.isObject(obj))
            return false;
        try {
            if (testObject(obj) === false)
                return false;
        }
        catch (e) {
            return false;
        }
        var key;
        for (key in obj) { }
        return key === undefined || core_hasOwn.call(obj, key);
    };
    Lib.isArray = function (value) {
        return Array.isArray ? Array.isArray(value) : Lib.isType("Array", value);
    };
    Lib.isWindow = function (obj) { return !!(obj && obj == obj.window); };
    Lib.isElement = function (obj) { var t = obj && (obj.ownerDocument || obj).documentElement; return t ? true : false; };
    Lib.trim = function (str, newline) {
        if (str) {
            return newline ? str.replace(/^(?:\s|\u3000|\ue4c6|\n|\r)*|(?:\s|\u3000|\ue4c6|\n|\r)*$/g, '') :
                str.replace(/^(?:\s|\u3000|\ue4c6)*|(?:\s|\u3000|\ue4c6)*$/g, '');
        }
        else {
            return '';
        }
    };
    Lib.replaceAll = function (s, str, repl, flags) {
        if (flags === void 0) { flags = "g"; }
        if (Lib.isNullEmpty(s) || Lib.isNullEmpty(str))
            return s;
        str = str.replace(/([^A-Za-z0-9 ])/g, "\\$1");
        s = s.replace(new RegExp(str, flags), repl);
        return s;
    };
    Lib.inArray = function (list, p, thisArg) {
        if (thisArg === void 0) { thisArg = null; }
        var isF = Lib.isFunction(p), index = -1;
        Lib.each(list, function (item, idx) {
            var ok = isF ? p.call(thisArg, item, idx) : (item == p);
            if (ok) {
                index = idx;
                return false;
            }
        }, thisArg);
        return index;
    };
    Lib.toArray = function (p, start, count) {
        if (start === void 0) { start = 0; }
        if (count === void 0) { count = Number.MAX_VALUE; }
        return p ? slice.apply(p, [start, count]) : p;
    };
    Lib.arrayToObject = function (array, fieldName) {
        var obj = {};
        Lib.each(array, function (item, index) {
            obj[item[fieldName]] = item;
        });
        return obj;
    };
    Lib.each = function (list, fn, thisArg) {
        if (thisArg === void 0) { thisArg = null; }
        if (!list)
            return;
        var len = list.length;
        for (var i = 0, len_1 = list.length; i < len_1; i++) {
            if (fn.call(thisArg, list[i], i) === false)
                break;
        }
    };
    Lib.eachProp = function (obj, callback, thisArg) {
        if (thisArg === void 0) { thisArg = null; }
        if (!obj)
            return;
        var item;
        for (var n in obj) {
            if (Lib.hasOwnProp(obj, n)) {
                item = obj[n];
                if (callback.call(thisArg, item, n) === false)
                    break;
            }
        }
    };
    /**
     * 扩展obj, 返回obj
     * @param obj 扩展到Obj
     * @param args 扩展源
     */
    Lib.extend = function (obj) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        if (obj) {
            Lib.each(args, function (p) {
                p && Lib.eachProp(p, function (item, name) { obj[name] = item; });
            });
        }
        return obj;
    };
    /**
     * 同步附值, 返回dest
     * @param dest 附值到
     * @param from 附值来源
     * @example syncPropertys(new User(), {name:'aaa'});
     */
    Lib.syncPropertys = function (dest, from) {
        if (from) {
            Lib.eachProp(from, function (item, key) {
                Lib.syncProperty(dest, key, from);
            });
        }
        return dest;
    };
    /**
     * 同步附值, 返回dest
     * @param dest 附值到
     * @param destKey
     * @param from 附值来源
     * @param fromKey
     * @example syncProperty(new User(), 'name', obj, 'myName');
     */
    Lib.syncProperty = function (dest, destKey, from, fromKey) {
        fromKey || (fromKey = destKey);
        dest[destKey] = from[fromKey];
        Object.defineProperty(from, fromKey, {
            enumerable: true, configurable: true,
            get: function () {
                return dest[destKey];
            },
            set: function (value) {
                dest[destKey] = value;
            }
        });
        return dest;
    };
    Lib.makeAutoId = function () {
        var t = new Date().valueOf();
        if ((++_tick) > 100000)
            _tick = 0;
        return [t, _tick].join('_');
    };
    Lib.format = function (str) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        Lib.each(args, function (item, index) {
            str = Lib.replaceAll(str, '{' + index + '}', item);
        });
        return str;
    };
    Lib.formatObject = function (str) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        Lib.each(args, function (item, index) {
            str = Lib.replaceAll(str, '{' + index + '}', item);
        });
        return str;
    };
    /**
     * 是否类
     * @param p 参数
     * @param cls 类
     */
    Lib.isClass = function (p, cls) {
        return p ? (p == cls || (p.prototype && p.prototype instanceof cls)) : false;
    };
    Lib.offset = function (element, offset) {
        if (offset) {
            var curOffset = Lib.offset(element);
            return {
                top: offset.top - curOffset.top,
                left: offset.left - curOffset.left
            };
        }
        else {
            var box = void 0;
            if (element.getBoundingClientRect)
                box = element.getBoundingClientRect();
            var win = window, docElem = document.documentElement;
            return {
                top: box.top + (win.pageYOffset || docElem.scrollTop) - (docElem.clientTop || 0),
                left: box.left + (win.pageXOffset || docElem.scrollLeft) - (docElem.clientLeft || 0)
            };
        }
    };
    Lib.stringEmpty = stringEmpty;
    return Lib;
}());
exports.Lib = Lib;
var _tick = 0;
//# sourceMappingURL=lib.js.map