"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var lib_1 = require("./lib");
var undef;
exports.breakOff = function (callback) { return function (source) {
    return new rxjs_1.Observable(function (observer) {
        source.subscribe({
            next: function (r) {
                !callback() && observer.next(r);
            },
            error: function (r) {
                !callback() && observer.error(r);
            },
            complete: function () {
                !callback() && observer.complete();
            }
        });
    });
}; };
exports.cacheToObject = function (obj, key) {
    var _key = key;
    return function (source) {
        return new rxjs_1.Observable(function (observer) {
            var key = obj ? (lib_1.Lib.isObject(_key) ? JSON.stringify(_key) : _key) : '';
            if (!key) {
                observer.error('cacheToObject key empty');
                observer.complete();
                return;
            }
            var setObjectCache = function (value, success, closed) {
                var cache = _getObjectCache(obj, key);
                var has = !!cache;
                has || (cache = { fns: [] });
                cache.closed = closed;
                cache.success = success;
                cache.value = value;
                has || _setObjectCache(obj, key, cache);
                if (closed) {
                    var fns = cache.fns;
                    cache.fns = [];
                    fns.forEach(function (item) { return item(cache); });
                }
            };
            var subFn = function () {
                setObjectCache(null, false, false);
                source.subscribe({
                    next: function (rs) {
                        observer.next(rs);
                        setObjectCache(rs, true, true);
                    },
                    error: function (err) {
                        observer.error(err);
                        setObjectCache(err, true, true);
                    },
                    complete: function () { return observer.complete(); }
                });
            };
            var cache = _getObjectCache(obj, key);
            var cacheSubFn = function (cache) {
                if (cache.success)
                    observer.next(cache.value);
                else
                    observer.error(cache.value);
                observer.complete();
            };
            if (cache) {
                if (cache.closed) {
                    cacheSubFn(cache);
                }
                else {
                    cache.fns.push(cacheSubFn);
                }
            }
            else
                subFn();
        });
    };
};
var _makeKey = function (key) {
    return lib_1.Lib.isObject(key) ? JSON.stringify(key) : key;
}, _objectCaches = function (obj) {
    var name = '__objectcache__';
    return obj[name] || (obj[name] = []);
}, _objectCacheIndex = function (objcache, key) {
    var caches = objcache;
    return caches.findIndex(function (item) { return item.key == key; });
}, _getObjectCache = function (obj, key) {
    var objectCaches = _objectCaches(obj);
    key = _makeKey(key);
    var index = _objectCacheIndex(objectCaches, key);
    var cacheItem = index >= 0 ? objectCaches[index] : undef;
    return cacheItem && cacheItem.value;
}, _setObjectCache = function (obj, key, value) {
    var objectCaches = _objectCaches(obj);
    key = _makeKey(key);
    var cacheItem = {
        key: key,
        value: value
    };
    var index = _objectCacheIndex(objectCaches, key);
    if (index >= 0)
        objectCaches[index] = cacheItem;
    else
        objectCaches.push(cacheItem);
    return cacheItem;
};
//#endregion cacheToObject
//# sourceMappingURL=rxjs.js.map